import React, { useRef, useEffect } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function createXMLHttpRequest() {
    function MyXMLHttpRequest() {
        this.object = new window.XMLHttpRequest();
        const fakeRequest = this;
        this.object.onabort = function (ev) {
            if (typeof fakeRequest.onabort === 'function') {
                fakeRequest.onabort(ev);
            }
        };
        this.object.onerror = function (ev) {
            if (typeof fakeRequest.onerror === 'function') {
                fakeRequest.onerror(ev);
            }
        };
        this.object.onload = function (ev) {
            if (typeof fakeRequest.onload === 'function') {
                fakeRequest.onload(ev);
            }
        };
        this.object.onloadend = function (ev) {
            if (typeof fakeRequest.onloadend === 'function') {
                fakeRequest.onloadend(ev);
            }
        };
        this.object.onloadstart = function (ev) {
            if (typeof fakeRequest.onloadstart === 'function') {
                fakeRequest.onloadstart(ev);
            }
        };
        this.object.onprogress = function (ev) {
            if (typeof fakeRequest.onprogress === 'function') {
                fakeRequest.onprogress(ev);
            }
        };
        this.object.ontimeout = function (ev) {
            if (typeof fakeRequest.ontimeout === 'function') {
                fakeRequest.ontimeout(ev);
            }
        };
        this.object.onreadystatechange = function (ev) {
            fakeRequest.readyState = fakeRequest.object.readyState;
            fakeRequest.status = fakeRequest.object.status;
            fakeRequest.statusText = fakeRequest.object.statusText;
            fakeRequest.response = fakeRequest.object.response;
            if (fakeRequest.responseType === '') {
                fakeRequest.responseText = fakeRequest.object.responseText;
                fakeRequest.responseXML = fakeRequest.object.responseXML;
            }
            fakeRequest.responseURL = fakeRequest.object.responseURL;
            if (typeof fakeRequest.onreadystatechange === 'function') {
                fakeRequest.onreadystatechange(ev);
            }
        };
        this.onabort = null;
        this.onerror = null;
        this.onload = null;
        this.onloadend = null;
        this.onloadstart = null;
        this.onprogress = null;
        this.onreadystatechange = null;
        this.ontimeout = null;
        this.readyState = XMLHttpRequest.UNSENT;
        this.responseHeaders = {};
        this.response = '';
        this.responseText = '';
        this.responseType = '';
        this.responseURL = '';
        this.responseXML = '';
        this.status = 0;
        this.statusText = '';
        this.timeout = 0;
        this.upload = null;
        this.withCredentials = false;
    }
    MyXMLHttpRequest.prototype = {
        method: '',
        url: '',
        addEventListener: function (ev, f) {
            return this[`on${ev}`] = f;
        },
        open: function (method, url, async, user, password) {
            this.method = method;
            this.url = url;
            this.object.responseType = this.responseType;
            this.object.open(method, url, typeof async === 'boolean' ? async : true, typeof user === 'undefined' ? null : user, typeof password === 'undefined' ? null : password);
        },
        send: function (data) {
            this.object.responseType = this.responseType;
            let matched = false;
            const request = this;
            MyXMLHttpRequest.handlers.forEach(function (item) {
                if ((typeof item.url === 'string' && item.url === request.url) ||
                    (item.url instanceof RegExp && item.url.test(request.url))) {
                    if (typeof item.method !== 'string' || item.method.toUpperCase() === request.method.toUpperCase()) {
                        matched = true;
                        const handleResponse = function () {
                            return __awaiter(this, void 0, void 0, function* () {
                                if (typeof request.onloadstart === 'function') {
                                    request.onloadstart();
                                }
                                // prepare arguments
                                let args = [];
                                if (item.url instanceof RegExp) {
                                    // get the matches from the regular expression
                                    args = item.url.exec(request.url);
                                    args.push(data);
                                }
                                else {
                                    args = [request.url, data];
                                }
                                // set response headers
                                let responseHeaders = null;
                                if (typeof item.headers === 'function') {
                                    responseHeaders = yield item.headers.apply(null, [request, ...args]);
                                }
                                else if (typeof item.headers === 'object') {
                                    responseHeaders = item.headers;
                                }
                                if (responseHeaders && typeof responseHeaders === 'object') {
                                    for (const name in responseHeaders) {
                                        if (responseHeaders.hasOwnProperty(name)) {
                                            request.responseHeaders[name.toLowerCase()] = responseHeaders[name];
                                        }
                                    }
                                }
                                request.status = item.status;
                                request.statusText = item.statusText;
                                request.readyState = XMLHttpRequest.HEADERS_RECEIVED;
                                if (typeof request.onreadystatechange === 'function') {
                                    request.onreadystatechange();
                                }
                                request.readyState = XMLHttpRequest.LOADING;
                                if (typeof request.onreadystatechange === 'function') {
                                    request.onreadystatechange();
                                }
                                if (typeof item.response === 'string' || typeof item.response === 'function') {
                                    let response = '';
                                    if (typeof item.response === 'string') {
                                        response = item.response;
                                    }
                                    else {
                                        response = yield item.response.apply(null, [request, ...args]);
                                    }
                                    request.response = response;
                                    if (request.responseType === '' || request.responseType === 'text') {
                                        request.responseText = request.response;
                                    }
                                    if (request.responseType === 'document') {
                                        request.responseXML = request.response;
                                    }
                                    request.readyState = XMLHttpRequest.DONE;
                                    const progressEvent = {
                                        target: request
                                    };
                                    if (typeof request.onreadystatechange === 'function') {
                                        request.onreadystatechange(progressEvent);
                                    }
                                    if (typeof request.onload === 'function') {
                                        request.onload(progressEvent);
                                    }
                                    if (typeof request.onloadend === 'function') {
                                        request.onloadend(progressEvent);
                                    }
                                }
                                else if (typeof item.proxy === 'string' || typeof item.proxy === 'function') {
                                    let url = '';
                                    if (typeof item.proxy === 'string') {
                                        url = item.proxy;
                                    }
                                    else {
                                        url = yield item.proxy.apply(null, args);
                                    }
                                    if (typeof url === 'string' && url.length > 0) {
                                        const proxyRequest = new XMLHttpRequest();
                                        proxyRequest.onreadystatechange = function () {
                                            if (proxyRequest.readyState === XMLHttpRequest.DONE &&
                                                proxyRequest.status === 200) {
                                                request.status = item.status;
                                                request.statusText = item.statusText;
                                                request.response = proxyRequest.response;
                                                request.readyState = XMLHttpRequest.DONE;
                                                if (request.responseType === '' || request.responseType === 'text') {
                                                    request.responseText = proxyRequest.response;
                                                }
                                                if (request.responseType === 'document') {
                                                    request.responseXML = proxyRequest.response;
                                                }
                                                request.readyState = XMLHttpRequest.DONE;
                                                if (typeof request.onreadystatechange === 'function') {
                                                    request.onreadystatechange();
                                                }
                                                if (typeof request.onload === 'function') {
                                                    request.onload();
                                                }
                                                if (typeof request.onloadend === 'function') {
                                                    request.onloadend();
                                                }
                                            }
                                        };
                                        proxyRequest.open('GET', url);
                                        proxyRequest.send();
                                    }
                                }
                            });
                        };
                        if (typeof item.responseTime === 'number') {
                            setTimeout(handleResponse, item.responseTime);
                        }
                        else if (typeof item.responseTime === 'object' && item.responseTime.length === 2) {
                            setTimeout(handleResponse, Math.random() * (item.responseTime[1] - item.responseTime[0]) + item.responseTime[0]);
                        }
                        else {
                            handleResponse();
                        }
                    }
                }
            });
            if (!matched) {
                this.object.send(data);
            }
        },
        abort: function () {
            this.object.abort();
        },
        setRequestHeader: function (header, value) {
            this.object.setRequestHeader(header, value);
        },
        getResponseHeader: function (header) {
            if (this.responseHeaders.hasOwnProperty(header.toLowerCase())) {
                return this.responseHeaders[header.toLowerCase()];
            }
            return this.object.getResponseHeader(header);
        },
        getAllResponseHeaders: function () {
            return Object.entries(this.responseHeaders).map(x => x.join(": ")).join("\n");
        },
        overrideMimeType: function (mimeType) {
            this.object.overrideMimeType(mimeType);
        }
    };
    MyXMLHttpRequest.prototype.constructor = MyXMLHttpRequest;
    MyXMLHttpRequest.handlers = [];
    MyXMLHttpRequest.addHandler = function (handler) {
        MyXMLHttpRequest.handlers.push(handler);
    };
    MyXMLHttpRequest.UNSENT = XMLHttpRequest.UNSENT;
    MyXMLHttpRequest.OPENED = XMLHttpRequest.OPENED;
    MyXMLHttpRequest.LOADING = XMLHttpRequest.LOADING;
    MyXMLHttpRequest.HEADERS_RECEIVED = XMLHttpRequest.HEADERS_RECEIVED;
    MyXMLHttpRequest.DONE = XMLHttpRequest.DONE;
    return MyXMLHttpRequest;
}

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

/**
 * Check if we're required to add a port number.
 *
 * @see https://url.spec.whatwg.org/#default-port
 * @param {Number|String} port Port number we need to check
 * @param {String} protocol Protocol we need to check against.
 * @returns {Boolean} Is it a default port for the given protocol
 * @api private
 */
var requiresPort = function required(port, protocol) {
  protocol = protocol.split(':')[0];
  port = +port;

  if (!port) return false;

  switch (protocol) {
    case 'http':
    case 'ws':
    return port !== 80;

    case 'https':
    case 'wss':
    return port !== 443;

    case 'ftp':
    return port !== 21;

    case 'gopher':
    return port !== 70;

    case 'file':
    return false;
  }

  return port !== 0;
};

var has = Object.prototype.hasOwnProperty
  , undef;

/**
 * Decode a URI encoded string.
 *
 * @param {String} input The URI encoded string.
 * @returns {String|Null} The decoded string.
 * @api private
 */
function decode(input) {
  try {
    return decodeURIComponent(input.replace(/\+/g, ' '));
  } catch (e) {
    return null;
  }
}

/**
 * Simple query string parser.
 *
 * @param {String} query The query string that needs to be parsed.
 * @returns {Object}
 * @api public
 */
function querystring(query) {
  var parser = /([^=?&]+)=?([^&]*)/g
    , result = {}
    , part;

  while (part = parser.exec(query)) {
    var key = decode(part[1])
      , value = decode(part[2]);

    //
    // Prevent overriding of existing properties. This ensures that build-in
    // methods like `toString` or __proto__ are not overriden by malicious
    // querystrings.
    //
    // In the case if failed decoding, we want to omit the key/value pairs
    // from the result.
    //
    if (key === null || value === null || key in result) continue;
    result[key] = value;
  }

  return result;
}

/**
 * Transform a query string to an object.
 *
 * @param {Object} obj Object that should be transformed.
 * @param {String} prefix Optional prefix.
 * @returns {String}
 * @api public
 */
function querystringify(obj, prefix) {
  prefix = prefix || '';

  var pairs = []
    , value
    , key;

  //
  // Optionally prefix with a '?' if needed
  //
  if ('string' !== typeof prefix) prefix = '?';

  for (key in obj) {
    if (has.call(obj, key)) {
      value = obj[key];

      //
      // Edge cases where we actually want to encode the value to an empty
      // string instead of the stringified value.
      //
      if (!value && (value === null || value === undef || isNaN(value))) {
        value = '';
      }

      key = encodeURIComponent(key);
      value = encodeURIComponent(value);

      //
      // If we failed to encode the strings, we should bail out as we don't
      // want to add invalid strings to the query.
      //
      if (key === null || value === null) continue;
      pairs.push(key +'='+ value);
    }
  }

  return pairs.length ? prefix + pairs.join('&') : '';
}

//
// Expose the module.
//
var stringify = querystringify;
var parse = querystring;

var querystringify_1 = {
	stringify: stringify,
	parse: parse
};

var slashes = /^[A-Za-z][A-Za-z0-9+-.]*:\/\//
  , protocolre = /^([a-z][a-z0-9.+-]*:)?(\/\/)?([\S\s]*)/i
  , whitespace = '[\\x09\\x0A\\x0B\\x0C\\x0D\\x20\\xA0\\u1680\\u180E\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200A\\u202F\\u205F\\u3000\\u2028\\u2029\\uFEFF]'
  , left = new RegExp('^'+ whitespace +'+');

/**
 * Trim a given string.
 *
 * @param {String} str String to trim.
 * @public
 */
function trimLeft(str) {
  return (str ? str : '').toString().replace(left, '');
}

/**
 * These are the parse rules for the URL parser, it informs the parser
 * about:
 *
 * 0. The char it Needs to parse, if it's a string it should be done using
 *    indexOf, RegExp using exec and NaN means set as current value.
 * 1. The property we should set when parsing this value.
 * 2. Indication if it's backwards or forward parsing, when set as number it's
 *    the value of extra chars that should be split off.
 * 3. Inherit from location if non existing in the parser.
 * 4. `toLowerCase` the resulting value.
 */
var rules = [
  ['#', 'hash'],                        // Extract from the back.
  ['?', 'query'],                       // Extract from the back.
  function sanitize(address) {          // Sanitize what is left of the address
    return address.replace('\\', '/');
  },
  ['/', 'pathname'],                    // Extract from the back.
  ['@', 'auth', 1],                     // Extract from the front.
  [NaN, 'host', undefined, 1, 1],       // Set left over value.
  [/:(\d+)$/, 'port', undefined, 1],    // RegExp the back.
  [NaN, 'hostname', undefined, 1, 1]    // Set left over.
];

/**
 * These properties should not be copied or inherited from. This is only needed
 * for all non blob URL's as a blob URL does not include a hash, only the
 * origin.
 *
 * @type {Object}
 * @private
 */
var ignore = { hash: 1, query: 1 };

/**
 * The location object differs when your code is loaded through a normal page,
 * Worker or through a worker using a blob. And with the blobble begins the
 * trouble as the location object will contain the URL of the blob, not the
 * location of the page where our code is loaded in. The actual origin is
 * encoded in the `pathname` so we can thankfully generate a good "default"
 * location from it so we can generate proper relative URL's again.
 *
 * @param {Object|String} loc Optional default location object.
 * @returns {Object} lolcation object.
 * @public
 */
function lolcation(loc) {
  var globalVar;

  if (typeof window !== 'undefined') globalVar = window;
  else if (typeof commonjsGlobal !== 'undefined') globalVar = commonjsGlobal;
  else if (typeof self !== 'undefined') globalVar = self;
  else globalVar = {};

  var location = globalVar.location || {};
  loc = loc || location;

  var finaldestination = {}
    , type = typeof loc
    , key;

  if ('blob:' === loc.protocol) {
    finaldestination = new Url(unescape(loc.pathname), {});
  } else if ('string' === type) {
    finaldestination = new Url(loc, {});
    for (key in ignore) delete finaldestination[key];
  } else if ('object' === type) {
    for (key in loc) {
      if (key in ignore) continue;
      finaldestination[key] = loc[key];
    }

    if (finaldestination.slashes === undefined) {
      finaldestination.slashes = slashes.test(loc.href);
    }
  }

  return finaldestination;
}

/**
 * @typedef ProtocolExtract
 * @type Object
 * @property {String} protocol Protocol matched in the URL, in lowercase.
 * @property {Boolean} slashes `true` if protocol is followed by "//", else `false`.
 * @property {String} rest Rest of the URL that is not part of the protocol.
 */

/**
 * Extract protocol information from a URL with/without double slash ("//").
 *
 * @param {String} address URL we want to extract from.
 * @return {ProtocolExtract} Extracted information.
 * @private
 */
function extractProtocol(address) {
  address = trimLeft(address);
  var match = protocolre.exec(address);

  return {
    protocol: match[1] ? match[1].toLowerCase() : '',
    slashes: !!match[2],
    rest: match[3]
  };
}

/**
 * Resolve a relative URL pathname against a base URL pathname.
 *
 * @param {String} relative Pathname of the relative URL.
 * @param {String} base Pathname of the base URL.
 * @return {String} Resolved pathname.
 * @private
 */
function resolve(relative, base) {
  if (relative === '') return base;

  var path = (base || '/').split('/').slice(0, -1).concat(relative.split('/'))
    , i = path.length
    , last = path[i - 1]
    , unshift = false
    , up = 0;

  while (i--) {
    if (path[i] === '.') {
      path.splice(i, 1);
    } else if (path[i] === '..') {
      path.splice(i, 1);
      up++;
    } else if (up) {
      if (i === 0) unshift = true;
      path.splice(i, 1);
      up--;
    }
  }

  if (unshift) path.unshift('');
  if (last === '.' || last === '..') path.push('');

  return path.join('/');
}

/**
 * The actual URL instance. Instead of returning an object we've opted-in to
 * create an actual constructor as it's much more memory efficient and
 * faster and it pleases my OCD.
 *
 * It is worth noting that we should not use `URL` as class name to prevent
 * clashes with the global URL instance that got introduced in browsers.
 *
 * @constructor
 * @param {String} address URL we want to parse.
 * @param {Object|String} [location] Location defaults for relative paths.
 * @param {Boolean|Function} [parser] Parser for the query string.
 * @private
 */
function Url(address, location, parser) {
  address = trimLeft(address);

  if (!(this instanceof Url)) {
    return new Url(address, location, parser);
  }

  var relative, extracted, parse, instruction, index, key
    , instructions = rules.slice()
    , type = typeof location
    , url = this
    , i = 0;

  //
  // The following if statements allows this module two have compatibility with
  // 2 different API:
  //
  // 1. Node.js's `url.parse` api which accepts a URL, boolean as arguments
  //    where the boolean indicates that the query string should also be parsed.
  //
  // 2. The `URL` interface of the browser which accepts a URL, object as
  //    arguments. The supplied object will be used as default values / fall-back
  //    for relative paths.
  //
  if ('object' !== type && 'string' !== type) {
    parser = location;
    location = null;
  }

  if (parser && 'function' !== typeof parser) parser = querystringify_1.parse;

  location = lolcation(location);

  //
  // Extract protocol information before running the instructions.
  //
  extracted = extractProtocol(address || '');
  relative = !extracted.protocol && !extracted.slashes;
  url.slashes = extracted.slashes || relative && location.slashes;
  url.protocol = extracted.protocol || location.protocol || '';
  address = extracted.rest;

  //
  // When the authority component is absent the URL starts with a path
  // component.
  //
  if (!extracted.slashes) instructions[3] = [/(.*)/, 'pathname'];

  for (; i < instructions.length; i++) {
    instruction = instructions[i];

    if (typeof instruction === 'function') {
      address = instruction(address);
      continue;
    }

    parse = instruction[0];
    key = instruction[1];

    if (parse !== parse) {
      url[key] = address;
    } else if ('string' === typeof parse) {
      if (~(index = address.indexOf(parse))) {
        if ('number' === typeof instruction[2]) {
          url[key] = address.slice(0, index);
          address = address.slice(index + instruction[2]);
        } else {
          url[key] = address.slice(index);
          address = address.slice(0, index);
        }
      }
    } else if ((index = parse.exec(address))) {
      url[key] = index[1];
      address = address.slice(0, index.index);
    }

    url[key] = url[key] || (
      relative && instruction[3] ? location[key] || '' : ''
    );

    //
    // Hostname, host and protocol should be lowercased so they can be used to
    // create a proper `origin`.
    //
    if (instruction[4]) url[key] = url[key].toLowerCase();
  }

  //
  // Also parse the supplied query string in to an object. If we're supplied
  // with a custom parser as function use that instead of the default build-in
  // parser.
  //
  if (parser) url.query = parser(url.query);

  //
  // If the URL is relative, resolve the pathname against the base URL.
  //
  if (
      relative
    && location.slashes
    && url.pathname.charAt(0) !== '/'
    && (url.pathname !== '' || location.pathname !== '')
  ) {
    url.pathname = resolve(url.pathname, location.pathname);
  }

  //
  // We should not add port numbers if they are already the default port number
  // for a given protocol. As the host also contains the port number we're going
  // override it with the hostname which contains no port number.
  //
  if (!requiresPort(url.port, url.protocol)) {
    url.host = url.hostname;
    url.port = '';
  }

  //
  // Parse down the `auth` for the username and password.
  //
  url.username = url.password = '';
  if (url.auth) {
    instruction = url.auth.split(':');
    url.username = instruction[0] || '';
    url.password = instruction[1] || '';
  }

  url.origin = url.protocol && url.host && url.protocol !== 'file:'
    ? url.protocol +'//'+ url.host
    : 'null';

  //
  // The href is just the compiled result.
  //
  url.href = url.toString();
}

/**
 * This is convenience method for changing properties in the URL instance to
 * insure that they all propagate correctly.
 *
 * @param {String} part          Property we need to adjust.
 * @param {Mixed} value          The newly assigned value.
 * @param {Boolean|Function} fn  When setting the query, it will be the function
 *                               used to parse the query.
 *                               When setting the protocol, double slash will be
 *                               removed from the final url if it is true.
 * @returns {URL} URL instance for chaining.
 * @public
 */
function set(part, value, fn) {
  var url = this;

  switch (part) {
    case 'query':
      if ('string' === typeof value && value.length) {
        value = (fn || querystringify_1.parse)(value);
      }

      url[part] = value;
      break;

    case 'port':
      url[part] = value;

      if (!requiresPort(value, url.protocol)) {
        url.host = url.hostname;
        url[part] = '';
      } else if (value) {
        url.host = url.hostname +':'+ value;
      }

      break;

    case 'hostname':
      url[part] = value;

      if (url.port) value += ':'+ url.port;
      url.host = value;
      break;

    case 'host':
      url[part] = value;

      if (/:\d+$/.test(value)) {
        value = value.split(':');
        url.port = value.pop();
        url.hostname = value.join(':');
      } else {
        url.hostname = value;
        url.port = '';
      }

      break;

    case 'protocol':
      url.protocol = value.toLowerCase();
      url.slashes = !fn;
      break;

    case 'pathname':
    case 'hash':
      if (value) {
        var char = part === 'pathname' ? '/' : '#';
        url[part] = value.charAt(0) !== char ? char + value : value;
      } else {
        url[part] = value;
      }
      break;

    default:
      url[part] = value;
  }

  for (var i = 0; i < rules.length; i++) {
    var ins = rules[i];

    if (ins[4]) url[ins[1]] = url[ins[1]].toLowerCase();
  }

  url.origin = url.protocol && url.host && url.protocol !== 'file:'
    ? url.protocol +'//'+ url.host
    : 'null';

  url.href = url.toString();

  return url;
}

/**
 * Transform the properties back in to a valid and full URL string.
 *
 * @param {Function} stringify Optional query stringify function.
 * @returns {String} Compiled version of the URL.
 * @public
 */
function toString(stringify) {
  if (!stringify || 'function' !== typeof stringify) stringify = querystringify_1.stringify;

  var query
    , url = this
    , protocol = url.protocol;

  if (protocol && protocol.charAt(protocol.length - 1) !== ':') protocol += ':';

  var result = protocol + (url.slashes ? '//' : '');

  if (url.username) {
    result += url.username;
    if (url.password) result += ':'+ url.password;
    result += '@';
  }

  result += url.host + url.pathname;

  query = 'object' === typeof url.query ? stringify(url.query) : url.query;
  if (query) result += '?' !== query.charAt(0) ? '?'+ query : query;

  if (url.hash) result += url.hash;

  return result;
}

Url.prototype = { set: set, toString: toString };

//
// Expose the URL parser and some additional properties that might be useful for
// others or testing.
//
Url.extractProtocol = extractProtocol;
Url.location = lolcation;
Url.trimLeft = trimLeft;
Url.qs = querystringify_1;

var urlParse = Url;

// tslint:disable:object-literal-sort-keys
/*
 * https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent
 */
const CLOSE_CODES = {
    CLOSE_NORMAL: 1000,
    CLOSE_GOING_AWAY: 1001,
    CLOSE_PROTOCOL_ERROR: 1002,
    CLOSE_UNSUPPORTED: 1003,
    CLOSE_NO_STATUS: 1005,
    CLOSE_ABNORMAL: 1006,
    UNSUPPORTED_DATA: 1007,
    POLICY_VIOLATION: 1008,
    CLOSE_TOO_LARGE: 1009,
    MISSING_EXTENSION: 1010,
    INTERNAL_ERROR: 1011,
    SERVICE_RESTART: 1012,
    TRY_AGAIN_LATER: 1013,
    TLS_HANDSHAKE: 1015,
};
const ERROR_PREFIX = {
    CONSTRUCTOR_ERROR: "Failed to construct 'WebSocket':",
    EVENT: {
        CONSTRUCT: "Failed to construct 'Event':",
        MESSAGE: "Failed to construct 'MessageEvent':",
        CLOSE: "Failed to construct 'CloseEvent':",
    },
};

class Event {
    constructor(type, eventInit = {}) {
        this.srcElement = null;
        this.NONE = Event.NONE;
        this.CAPTURING_PHASE = Event.CAPTURING_PHASE;
        this.AT_TARGET = Event.AT_TARGET;
        this.BUBBLING_PHASE = Event.BUBBLING_PHASE;
        if (!type) {
            throw new TypeError(`${ERROR_PREFIX.EVENT.CONSTRUCT} 1 argument required, but only 0 present.`);
        }
        if (typeof eventInit !== 'object') {
            throw new TypeError(`${ERROR_PREFIX.EVENT.CONSTRUCT} parameter 2 ('eventInitDict') is not an object.`);
        }
        this._type = String(type);
        this.timeStamp = Date.now();
        this._target = null;
        this.returnValue = true;
        this.isTrusted = false;
        this.eventPhase = 0;
        this.defaultPrevented = false;
        this._currentTarget = null;
        this._cancelable = eventInit.cancelable ? true : false;
        this.cancelBubble = false;
        this._bubbles = eventInit.bubbles ? true : false;
        this.composed = eventInit.composed ? true : false;
    }
    get type() {
        return this._type;
    }
    get bubbles() {
        return this._bubbles;
    }
    get cancelable() {
        return this._cancelable;
    }
    get target() {
        return this._target;
    }
    get currentTarget() {
        return this._currentTarget;
    }
    initEvent(type, bubbles, cancelable) {
        this._type = String(type);
        this._bubbles = !!bubbles;
        this._cancelable = !!cancelable;
    }
    composedPath() {
        return [];
    }
    preventDefault() {
        // TODO
    }
    stopImmediatePropagation() {
        // TODO
    }
    stopPropagation() {
        // TODO
    }
    _setTarget(target) {
        this._target = target;
        this._currentTarget = target;
    }
}
Event.NONE = 0;
Event.CAPTURING_PHASE = 1;
Event.AT_TARGET = 2;
Event.BUBBLING_PHASE = 3;

class CloseEvent extends Event {
    constructor(type, eventInit = {}) {
        super(type, eventInit);
        this.code = eventInit.code ? Number(eventInit.code) : 0;
        this.reason = eventInit.reason ? String(eventInit.reason) : '';
        this.wasClean = eventInit.wasClean ? true : false;
    }
}

class MessageEvent extends Event {
    constructor(type, eventInit = {}) {
        super(type, eventInit);
        // TODO!
        this.ports = [];
        // TODO!
        this.source = null;
        this.data = eventInit.hasOwnProperty('data') ? eventInit.data : null;
        this.origin = eventInit.hasOwnProperty('origin') ? String(eventInit.origin) : '';
        this.lastEventId = eventInit.hasOwnProperty('lastEventId') ? String(eventInit.lastEventId) : '';
    }
}

/*
 * Creates an Event object and extends it to allow full modification of
 * its properties.
 *
 * @param {object} config - within config you will need to pass type and optionally target
 */
function createEvent(config) {
    const eventObject = new Event(config.type);
    if (config.target) {
        eventObject._setTarget(config.target);
    }
    return eventObject;
}
/*
 * Creates a MessageEvent object and extends it to allow full modification of
 * its properties.
 *
 * @param {object} config - within config: type, origin, data and optionally target
 */
function createMessageEvent(config) {
    const { type, origin, data, target } = config;
    const messageEvent = new MessageEvent(type, { data, origin });
    if (target) {
        messageEvent._setTarget(target);
    }
    return messageEvent;
}
/*
 * Creates a CloseEvent object and extends it to allow full modification of
 * its properties.
 *
 * @param {object} config - within config: type and optionally target, code, and reason
 */
function createCloseEvent(config) {
    const { code, reason, type, target } = config;
    let { wasClean } = config;
    if (!wasClean) {
        wasClean = code === 1000;
    }
    const closeEvent = new CloseEvent(type, { code, reason, wasClean });
    if (target) {
        closeEvent._setTarget(target);
    }
    return closeEvent;
}

function reject(array, callback) {
    const results = [];
    array.forEach((itemInArray) => {
        if (!callback(itemInArray)) {
            results.push(itemInArray);
        }
    });
    return results;
}
function filter(array, callback) {
    const results = [];
    array.forEach((itemInArray) => {
        if (callback(itemInArray)) {
            results.push(itemInArray);
        }
    });
    return results;
}

function retrieveGlobalObject() {
    if (typeof window !== 'undefined') {
        return window;
    }
    return typeof process === 'object' && typeof require === 'function' && typeof global === 'object' ? global : this;
}

/**
 * The network bridge is a way for the mock websocket object to 'communicate' with
 * all available servers. This is a singleton object so it is important that you
 * clean up urlMap whenever you are finished.
 */
class NetworkBridge {
    constructor() {
        this.urlMap = {};
    }
    /**
     * Attaches a websocket object to the urlMap hash so that it can find the server
     * it is connected to and the server in turn can find it.
     *
     * @param {object} websocket - websocket object to add to the urlMap hash
     * @param {string} url
     */
    attachWebSocket(websocket, url) {
        const connectionLookup = this.urlMap[url];
        if (!connectionLookup || !connectionLookup.server || connectionLookup.websockets.indexOf(websocket) !== -1) {
            return undefined;
        }
        connectionLookup.websockets.push(websocket);
        return connectionLookup.server;
    }
    /**
     * Attaches a server object to the urlMap hash so that it can find a websockets
     * which are connected to it and so that websockets can in turn can find it.
     *
     * @param {object} server - server object to add to the urlMap hash
     * @param {string} url
     */
    attachServer(server, url) {
        const connectionLookup = this.urlMap[url];
        if (connectionLookup) {
            // throw new Error('Can not attach server. Already exists')
            return undefined;
        }
        this.urlMap[url] = { server, websockets: [] };
        return server;
    }
    /**
     * Finds the server which is 'running' on the given url.
     *
     * @param {string} url - the url to use to find which server is running on it
     */
    serverLookup(url) {
        const connectionLookup = this.urlMap[url];
        return connectionLookup ? connectionLookup.server : undefined;
    }
    /**
     * Finds all websockets which is 'listening' on the given url.
     *
     * @param {string} url - the url to use to find all websockets which are associated with it
     */
    websocketsLookup(url) {
        const connectionLookup = this.urlMap[url];
        return connectionLookup ? connectionLookup.websockets : [];
    }
    /**
     * Removes the entry associated with the url.
     *
     * @param {string} url
     */
    removeServer(url) {
        delete this.urlMap[url];
    }
    /**
     * Removes the individual websocket from the map of associated websockets.
     *
     * @param {object} websocket - websocket object to remove from the url map
     * @param {string} url
     */
    removeWebSocket(websocket, url) {
        const connectionLookup = this.urlMap[url];
        if (connectionLookup) {
            connectionLookup.websockets = reject(connectionLookup.websockets, (socket) => socket === websocket);
        }
    }
}
var networkBridge = new NetworkBridge(); // Note: this is a singleton

/*
 * EventTarget is an interface implemented by objects that can
 * receive events and may have listeners for them.
 *
 * https://developer.mozilla.org/en-US/docs/Web/API/EventTarget
 */
class EventTarget {
    constructor() {
        this.listeners = {};
    }
    addEventListener(type, listener, options) {
        if (typeof listener === 'function') {
            if (!Array.isArray(this.listeners[type])) {
                this.listeners[type] = [];
            }
            // Only add the same function once
            if (filter(this.listeners[type], (item) => item === listener).length === 0) {
                this.listeners[type].push(listener);
            }
        }
    }
    removeEventListener(type, callback, options) {
        const arrayOfListeners = this.listeners[type];
        this.listeners[type] = reject(arrayOfListeners, (listener) => listener === callback);
    }
    dispatchEvent(event, ...customArguments) {
        const eventName = event.type;
        const listeners = this.listeners[eventName];
        if (!Array.isArray(listeners)) {
            return false;
        }
        listeners.forEach((listener) => {
            if (customArguments.length > 0) {
                listener.apply(this, customArguments);
            }
            else {
                listener.call(this, event);
            }
        });
        return true;
    }
}

/**
 * This delay allows the thread to finish assigning its on* methods
 * before invoking the delay callback.
 *
 * @param callback the callback which will be invoked after the timeout
 * @param context the context in which to invoke the function
 */
function delay(callback, context) {
    setImmediate((ctx) => callback.call(ctx), context);
}

function logError(message) {
    if (typeof process !== 'undefined' && process.env.NODE_ENV !== 'test') {
        // tslint:disable-next-line:no-console
        console.error(message);
    }
}

/**
 * The main websocket class which is designed to mimick the native WebSocket class as close
 * as possible.
 */
class WebSocket extends EventTarget {
    /**
     * @param {string} url
     */
    constructor(url, protocol = '') {
        super();
        this.binaryType = 'blob';
        this.CLOSED = WebSocket.CLOSED;
        this.CLOSING = WebSocket.CLOSING;
        this.CONNECTING = WebSocket.CONNECTING;
        this.OPEN = WebSocket.OPEN;
        this._bufferedAmount = 0;
        this._extensions = '';
        this._protocol = '';
        this._readyState = WebSocket.CONNECTING;
        this._url = '';
        this._onclose = null;
        this._onerror = null;
        this._onmessage = null;
        this._onopen = null;
        if (!url) {
            throw new TypeError(`${ERROR_PREFIX.CONSTRUCTOR_ERROR} 1 argument required, but only 0 present.`);
        }
        const urlRecord = new urlParse(url);
        if (urlRecord.protocol !== 'ws:' && urlRecord.protocol !== 'wss:') {
            throw new Error('SyntaxError: url scheme incorrect');
        }
        if (urlRecord.hash) {
            throw new Error('SyntaxError: url fragment exists');
        }
        if (!urlRecord.pathname) {
            urlRecord.set('pathname', '/');
        }
        this._url = urlRecord.toString();
        let protocols = [];
        if (typeof protocol === 'string') {
            this._protocol = protocol;
            protocols = [protocol];
        }
        else if (Array.isArray(protocol) && protocol.length > 0) {
            this._protocol = protocol[0];
            protocols = [...protocol];
        }
        const server = networkBridge.attachWebSocket(this, this.url);
        if (!server) {
            delay(function delayCallback() {
                this._readyState = WebSocket.CLOSED;
                this.dispatchEvent(createEvent({ type: 'error', target: this }));
                this.dispatchEvent(createCloseEvent({ type: 'close', target: this, code: CLOSE_CODES.CLOSE_NORMAL }));
                logError(`WebSocket connection to '${this.url}' failed`);
            }, this);
            return;
        }
        if (typeof server.options.verifyClient === 'function' && !server.options.verifyClient()) {
            delay(function delayCallback() {
                this._readyState = WebSocket.CLOSED;
                networkBridge.removeWebSocket(this, this.url);
                this.dispatchEvent(createEvent({ type: 'error', target: this }));
                this.dispatchEvent(createCloseEvent({ type: 'close', target: this, code: CLOSE_CODES.CLOSE_NORMAL }));
                logError(`WebSocket connection to '${this.url}' failed: HTTP Authentication failed; no valid credentials available`);
            }, this);
            return;
        }
        if (typeof server.options.selectProtocol === 'function') {
            const selected = server.options.selectProtocol(protocols);
            const isFilled = selected !== '';
            const isRequested = protocols.indexOf(selected) !== -1;
            if (isFilled && !isRequested) {
                delay(function delayCallback() {
                    this._readyState = WebSocket.CLOSED;
                    networkBridge.removeWebSocket(this, this.url);
                    this.dispatchEvent(createEvent({ type: 'error', target: this }));
                    this.dispatchEvent(createCloseEvent({ type: 'close', target: this, code: CLOSE_CODES.CLOSE_NORMAL }));
                    logError(`WebSocket connection to '${this.url}' failed: Invalid Sub-Protocol`);
                }, this);
                return;
            }
            this._protocol = selected;
        }
        delay(function delayCallback() {
            this._readyState = WebSocket.OPEN;
            this.dispatchEvent(createEvent({ type: 'open', target: this }));
            server.dispatchSocketEvent('connection', this);
        }, this);
    }
    get bufferedAmount() {
        return this._bufferedAmount;
    }
    get extensions() {
        return this._extensions;
    }
    get protocol() {
        return this._protocol;
    }
    get readyState() {
        return this._readyState;
    }
    get url() {
        return this._url;
    }
    get onclose() {
        return this._onclose;
    }
    set onclose(l) {
        if (this._onclose) {
            this.removeEventListener('close', this._onclose);
            this._onclose = null;
        }
        if (l) {
            this._onclose = l;
            this.addEventListener('close', l);
        }
    }
    get onerror() {
        return this._onerror;
    }
    set onerror(l) {
        if (this._onerror) {
            this.removeEventListener('error', this._onerror);
            this._onerror = null;
        }
        if (l) {
            this._onerror = l;
            this.addEventListener('error', l);
        }
    }
    get onmessage() {
        return this._onmessage;
    }
    set onmessage(l) {
        if (this._onmessage) {
            this.removeEventListener('message', this._onmessage);
            this._onmessage = null;
        }
        if (l) {
            this._onmessage = l;
            this.addEventListener('message', l);
        }
    }
    get onopen() {
        return this._onopen;
    }
    set onopen(l) {
        if (this._onopen) {
            this.removeEventListener('open', this._onopen);
            this._onopen = null;
        }
        if (l) {
            this._onopen = l;
            this.addEventListener('open', l);
        }
    }
    /**
     * Transmits data to the server over the WebSocket connection.
     *
     * https://developer.mozilla.org/en-US/docs/Web/API/WebSocket#send()
     */
    send(data) {
        if (this.readyState === WebSocket.CONNECTING) {
            networkBridge.removeWebSocket(this, this.url);
            throw new Error('InvalidStateError');
        }
        if (this.readyState === WebSocket.CLOSING || this.readyState === WebSocket.CLOSED) {
            this._bufferedAmount += data.length;
            return;
        }
        const server = networkBridge.serverLookup(this.url);
        if (server) {
            delay(() => {
                server.dispatchSocketEvent('message', this, data);
            }, server);
        }
    }
    /**
     * Closes the WebSocket connection or connection attempt, if any.
     * If the connection is already CLOSED, this method does nothing.
     *
     * https://developer.mozilla.org/en-US/docs/Web/API/WebSocket#close()
     */
    close(code, reason) {
        if (code && !(code === 1000 || (code >= 3000 && code < 5000))) {
            throw new Error('InvalidAccessError: close code out of user configurable range');
        }
        if (this.readyState === WebSocket.CLOSING || this.readyState === WebSocket.CLOSED) {
            return;
        }
        this._readyState = WebSocket.CLOSING;
        delay(function closeCallback() {
            if (!code)
                code = 1000;
            this._readyState = WebSocket.CLOSED;
            networkBridge.removeWebSocket(this, this.url);
            const closeEvent = createCloseEvent({ type: 'close', target: this, code, reason });
            this.dispatchEvent(closeEvent);
            const server = networkBridge.serverLookup(this.url);
            if (server) {
                server.dispatchSocketEvent('close', this);
            }
        }, this);
    }
    _moveToState(state) {
        this._readyState = state;
    }
}
WebSocket.CONNECTING = 0;
WebSocket.OPEN = 1;
WebSocket.CLOSING = 2;
WebSocket.CLOSED = 3;

/*
 * https://github.com/websockets/ws#simple-server
 */
class Server {
    /*
     * @param {string} url
     */
    constructor(url, options = {}) {
        this.listeners = new Map();
        const urlRecord = new urlParse(url);
        if (!urlRecord.pathname) {
            urlRecord.set('pathname', '/');
        }
        this.url = urlRecord.toString();
        this.originalWebSocket = null;
        const server = networkBridge.attachServer(this, this.url);
        if (!server) {
            this.dispatchError(new Error('A mock server is already listening on this url'));
        }
        if (typeof options.mockGlobal === 'undefined') {
            options.mockGlobal = true;
        }
        this.options = options;
        this.start();
    }
    /*
     * Attaches the mock websocket object to the global object
     */
    start() {
        if (!this.options.mockGlobal)
            return;
        const globalObj = retrieveGlobalObject();
        if (globalObj.WebSocket) {
            this.originalWebSocket = globalObj.WebSocket;
        }
        globalObj.WebSocket = WebSocket;
    }
    /*
     * Removes the mock websocket object from the global object
     */
    stop(callback) {
        if (this.options.mockGlobal) {
            const globalObj = retrieveGlobalObject();
            if (this.originalWebSocket) {
                globalObj.WebSocket = this.originalWebSocket;
            }
            else {
                delete globalObj.WebSocket;
            }
            this.originalWebSocket = null;
        }
        networkBridge.removeServer(this.url);
        if (typeof callback === 'function')
            callback();
    }
    /*
     * This is the main function for the mock server to subscribe to the on events.
     *
     * ie: mockServer.on('connection', function() { console.log('a mock client connected'); });
     *
     * @param {string} type - The event key to subscribe to. Valid keys are: connection, message, and close.
     * @param {function} callback - The callback which should be called when a certain event is fired.
     */
    on(type, cb) {
        if (typeof cb !== 'function')
            return;
        const listeners = this.listeners.get(type) || [];
        if (filter(listeners, (item) => item === cb).length === 0) {
            listeners.push(cb);
            this.listeners.set(type, listeners);
        }
    }
    dispatchError(error) {
        const listeners = this.listeners.get('error');
        if (!listeners)
            return;
        for (const l of listeners) {
            const cb = l;
            cb(error);
        }
    }
    dispatchSocketEvent(type, s, data) {
        const listeners = this.listeners.get(type);
        if (!listeners)
            return;
        for (const l of listeners) {
            switch (type) {
                case 'close': {
                    const cb = l;
                    cb(s);
                    break;
                }
                case 'connection': {
                    const cb = l;
                    if (!s)
                        throw new Error('no socket');
                    cb(s);
                    break;
                }
                case 'message': {
                    const cb = l;
                    if (!s)
                        throw new Error('no socket');
                    if (!data)
                        throw new Error('no data');
                    cb(s, data);
                    break;
                }
            }
        }
    }
    /*
     * This send function will notify all mock clients via their onmessage callbacks that the server
     * has a message for them.
     *
     * @param {*} data - Any javascript object which will be crafted into a MessageObject.
     */
    send(data, options = {}) {
        this.emit('message', data, options);
    }
    /*
     * Sends a generic message event to all mock clients.
     */
    emit(event, data, options = {}) {
        let websockets = options.websockets;
        if (!websockets) {
            websockets = networkBridge.websocketsLookup(this.url);
        }
        if (typeof options !== 'object' || arguments.length > 3) {
            data = Array.prototype.slice.call(arguments, 1, arguments.length);
        }
        websockets.forEach((socket) => {
            socket.dispatchEvent(createMessageEvent({ data, origin: this.url, target: socket, type: event }));
        });
    }
    /*
     * Closes the connection and triggers the onclose method of all listening
     * websockets. After that it removes itself from the urlMap so another server
     * could add itself to the url.
     *
     * @param {object} options
     */
    close(options = {}) {
        const { code, reason, wasClean } = options;
        const listeners = networkBridge.websocketsLookup(this.url);
        // Remove server before notifications to prevent immediate reconnects from
        // socket onclose handlers
        networkBridge.removeServer(this.url);
        listeners.forEach((socket) => {
            socket._moveToState(WebSocket.CLOSED);
            socket.dispatchEvent(createCloseEvent({
                code: code || CLOSE_CODES.CLOSE_NORMAL,
                reason: reason || '',
                target: socket,
                type: 'close',
                wasClean,
            }));
        });
        this.dispatchSocketEvent('close');
    }
    /*
     * Returns an array of websockets which are listening to this server
     */
    clients() {
        return networkBridge.websocketsLookup(this.url);
    }
    /*
     * Simulate an event from the server to the clients. Useful for
     * simulating errors.
     */
    simulate(event) {
        const listeners = networkBridge.websocketsLookup(this.url);
        if (event === 'error') {
            listeners.forEach((socket) => {
                socket._moveToState(WebSocket.CLOSED);
                socket.dispatchEvent(createEvent({ type: 'error' }));
            });
        }
    }
}

function mkUrl(...args) {
    return args.reduce((a, b) => new URL(b, a));
}

var defineLocalIframe = ({ fetch, getUrl }) => (props) => {
    var _a;
    const fetchUrlContent = (url, init) => fetch(url, init);
    let mockServers = [];
    (_a = props.webSocketSetup) === null || _a === void 0 ? void 0 : _a.call(props, url => {
        const mockServer = new Server(url, { mockGlobal: false });
        mockServers.push(mockServer);
        return mockServer;
    });
    const frame = useRef(null);
    const patchedElements = new WeakSet();
    const patchedElementSrcDocs = new WeakMap();
    function getResourceUrl(url, contextUrl) {
        const fullUrl = mkUrl(contextUrl, url);
        return getUrl(fullUrl);
    }
    function patchHtmlCode(htmlCode, contextUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const xmlDoc = new DOMParser().parseFromString(htmlCode, 'text/html');
            contextUrl = mkUrl(contextUrl, xmlDoc.baseURI.startsWith(window.location.origin) ? contextUrl : xmlDoc.baseURI);
            patchXmlImgTags(xmlDoc, contextUrl);
            patchXmlStyleTags(xmlDoc, contextUrl);
            yield patchXmlLinkTags(xmlDoc, contextUrl);
            patchXmlScriptTags(xmlDoc, contextUrl);
            patchXmlIframeTags(xmlDoc);
            return {
                html: `<!DOCTYPE html>${xmlDoc.documentElement.outerHTML}`,
                context: contextUrl.href || contextUrl
            };
        });
    }
    function patchCssUrls(cssCode, contextUrl) {
        return cssCode.replaceAll(/url\(["']?(.*?)["']?\)/gm, (m, url) => {
            return `url("${getResourceUrl(url, contextUrl)}")`;
        });
    }
    function patchXmlImgTags(xmlDoc, contextUrl) {
        for (const tag of xmlDoc.getElementsByTagName('img')) {
            const src = tag.getAttribute('src');
            if (src) {
                tag.setAttribute('src', getResourceUrl(src, contextUrl));
            }
        }
    }
    function patchXmlStyleTags(xmlDoc, contextUrl) {
        for (const tag of xmlDoc.getElementsByTagName('style')) {
            tag.innerHTML = patchCssUrls(tag.innerHTML, contextUrl);
        }
    }
    function getFrameForDocument(document) {
        const w = document.defaultView || document.parentWindow;
        const frames = w.parent.document.getElementsByTagName('iframe');
        for (let i = frames.length; i-- > 0;) {
            const frame = frames[i];
            try {
                const d = frame.contentDocument || frame.contentWindow.document;
                if (d === document)
                    return frame;
            }
            catch (e) { }
        }
    }
    function tryGetIframeContext(iframe) {
        if (!iframe)
            return null;
        const src = iframe.getAttribute('patched-src');
        if (src) {
            return src;
        }
        return tryGetIframeContext(getFrameForDocument(iframe.ownerDocument));
    }
    function patchLinkTag(tag, contextUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const rel = tag.getAttribute('rel');
            switch (rel) {
                case 'stylesheet':
                    {
                        const href = tag.getAttribute('href');
                        const hrefContext = mkUrl(contextUrl, href);
                        try {
                            const data = yield (yield fetchUrlContent(hrefContext.href, {
                                headers: {
                                    Accept: `text/css,*/*;q=0.1`,
                                    'Accept-Encoding': 'gzip, deflate, br'
                                }
                            })).text();
                            tag.outerHTML = `<style>${patchCssUrls(data, hrefContext)}</style>`;
                        }
                        catch (_a) { }
                    }
                    break;
                default: {
                    const href = tag.getAttribute('href');
                    if (href) {
                        tag.setAttribute('href', getResourceUrl(href, contextUrl));
                        tag.setAttribute('patched-href', href);
                    }
                }
            }
        });
    }
    function patchXmlLinkTags(xmlDoc, contextUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const tags = [...xmlDoc.getElementsByTagName('link')];
            for (const tag of tags) {
                yield patchLinkTag(tag, contextUrl);
            }
        });
    }
    function patchXmlScriptTags(xmlDoc, contextUrl) {
        for (const tag of xmlDoc.getElementsByTagName('script')) {
            const src = tag.getAttribute('src');
            if (src) {
                tag.setAttribute('src', getResourceUrl(src, contextUrl));
                tag.setAttribute('patched-src', src);
            }
        }
    }
    function patchXmlIframeTags(xmlDoc) {
        for (const tag of xmlDoc.getElementsByTagName('iframe')) {
            const src = tag.getAttribute('src');
            if (src) {
                tag.removeAttribute('src');
                tag.setAttribute('patched-src', src);
            }
        }
    }
    function proxySrc(src) {
        const url = new URL(src).href;
        return props.proxy(url);
    }
    function patchIframeConsole(iframe) {
        // The console may keep references to objects, preventing them from getting destroyed.
        // Solution - disable the console inside iframes.
        const contentWindow = iframe.contentWindow;
        contentWindow.console = new Proxy({}, {
            get() {
                return () => null;
            }
        });
    }
    function patchIframeClasses(iframe) {
        iframe.contentWindow.ArrayBuffer = ArrayBuffer;
    }
    function patchIframeEventListeners(iframe, context) {
        if (!props.onMessagePatchStrategy)
            return;
        if (typeof props.onMessagePatchStrategy == 'function') {
            return props.onMessagePatchStrategy(iframe, context);
        }
        if (props.onMessagePatchStrategy == 'patchedOriginClone') {
            const win = iframe.contentWindow;
            const prototype = win.EventTarget.prototype;
            const messageMap = new WeakMap();
            const _addEventListener = prototype.addEventListener;
            prototype.addEventListener = function (type, callback, options) {
                if (type == 'message') {
                    const newCallback = (evt) => {
                        if (callback == null)
                            return;
                        let origin = evt.origin;
                        try {
                            origin = new URL(context).origin;
                        }
                        catch (e) { }
                        // eslint-disable-next-line @typescript-eslint/no-explicit-any
                        evt = new evt.constructor(evt.type, new Proxy(evt, {
                            get: (target, prop) => (prop == 'origin' ? origin : target[prop])
                        }));
                        if ('handleEvent' in callback) {
                            return callback.handleEvent(evt);
                        }
                        else {
                            return callback(evt);
                        }
                    };
                    messageMap.set(callback, newCallback);
                    return _addEventListener.apply(this, [type, newCallback, options]);
                }
                else {
                    return _addEventListener.apply(this, [type, callback, options]);
                }
            };
            const _removeEventListener = prototype.removeEventListener;
            prototype.removeEventListener = function (type, callback, options) {
                if (type == 'message') {
                    const newCallback = messageMap.get(callback);
                    return _removeEventListener.apply(this, [type, newCallback, options]);
                }
                else {
                    return _removeEventListener.apply(this, [type, callback, options]);
                }
            };
        }
    }
    function patchIframePostMessage(iframe) {
        if (!props.postMessagePatchStrategy)
            return;
        if (!iframe.contentWindow)
            return;
        const win = iframe.contentWindow;
        if (typeof props.postMessagePatchStrategy == 'function') {
            return props.postMessagePatchStrategy(iframe);
        }
        if (props.postMessagePatchStrategy == 'target') {
            // Evaluating this code from the context of the iframe window
            // ensures that the Message event gets the correct source attribute.
            return win.eval(`
                    const _postMessage = window.postMessage;
                    window.postMessage = function(message, targetOrigin, transfer) {
                        return _postMessage.apply(this, [message, "*", transfer]);
                    }
                `);
        }
        if (props.postMessagePatchStrategy == 'top') {
            const oldPostMessage = win.postMessage.bind(win);
            win.postMessage = function myPostMessage(...args) {
                args[1] = '*';
                return oldPostMessage(...args);
            };
        }
    }
    function findDescriptor(obj, prop) {
        if (obj != null) {
            return Object.hasOwnProperty.call(obj, prop)
                ? Object.getOwnPropertyDescriptor(obj, prop)
                : findDescriptor(Object.getPrototypeOf(obj), prop);
        }
    }
    function addLocalUrlSetter(property, elem, context) {
        const { get, set } = findDescriptor(elem, property);
        Object.defineProperty(elem, property, {
            configurable: true,
            enumerable: true,
            get() {
                const v = get.call(this);
                return v;
            },
            set(v) {
                // modify value before applying it to the default setter
                set.call(this, getResourceUrl(v, context));
                elem.setAttribute(`patched-${property}`, v);
            }
        });
        const setAttribute = elem.setAttribute.bind(elem);
        elem.setAttribute = (qualifiedName, value) => {
            if (qualifiedName.toLowerCase() == property.toLowerCase()) {
                setAttribute(qualifiedName, getResourceUrl(value, context));
                setAttribute(`patched-${qualifiedName}`, value);
            }
            else {
                setAttribute(qualifiedName, value);
            }
        };
        const setAttributeNS = elem.setAttributeNS.bind(elem);
        elem.setAttributeNS = (namespace, qualifiedName, value) => {
            if (qualifiedName.toLowerCase() == property.toLowerCase()) {
                setAttributeNS(namespace, qualifiedName, getResourceUrl(value, context));
                setAttributeNS(namespace, `patched-${qualifiedName}`, value);
            }
            else {
                setAttributeNS(namespace, qualifiedName, value);
            }
        };
        const setAttributeNode = elem.setAttributeNode.bind(elem);
        elem.setAttributeNode = attr => {
            if (attr.name.toLowerCase() == property.toLowerCase()) {
                const patchedAttr = elem.ownerDocument.createAttribute(`patched-${attr.name}`);
                patchedAttr.value = attr.value;
                const newAttr = elem.ownerDocument.createAttribute(attr.name);
                newAttr.value = getResourceUrl(attr.value, context);
                setAttributeNode(patchedAttr);
                return setAttributeNode(newAttr);
            }
            else {
                return setAttributeNode(attr);
            }
        };
        const setAttributeNodeNS = elem.setAttributeNodeNS.bind(elem);
        elem.setAttributeNodeNS = attr => {
            if (attr.name.toLowerCase() == property.toLowerCase()) {
                const patchedAttr = elem.ownerDocument.createAttributeNS(attr.namespaceURI, `patched-${attr.name}`);
                patchedAttr.value = attr.value;
                const newAttr = elem.ownerDocument.createAttributeNS(attr.namespaceURI, attr.name);
                newAttr.value = getResourceUrl(attr.value, context);
                setAttributeNodeNS(patchedAttr);
                return setAttributeNodeNS(newAttr);
            }
            else {
                return setAttributeNodeNS(attr);
            }
        };
    }
    function patchIframeTags(iframe, context) {
        if (!props.tagPatchStrategy)
            return;
        if (typeof props.tagPatchStrategy == 'function') {
            return props.tagPatchStrategy(iframe, context);
        }
        if (props.tagPatchStrategy == 'createEl') {
            if (!iframe.contentWindow)
                return;
            const frameDoc = iframe.contentWindow.document;
            const createFrameElem = frameDoc.createElement.bind(frameDoc);
            const createFrameElemNS = frameDoc.createElementNS.bind(frameDoc);
            const patchElem = (tagName, elem) => {
                switch (tagName.toLowerCase()) {
                    case 'img':
                    case 'script':
                        addLocalUrlSetter('src', elem, context);
                        break;
                    case 'link':
                        addLocalUrlSetter('href', elem, context);
                        break;
                }
                return elem;
            };
            frameDoc.createElement = tagName => {
                const elem = createFrameElem(tagName);
                return patchElem(tagName, elem);
            };
            frameDoc.createElementNS = (nameSpace, tagName) => {
                const elem = createFrameElemNS(nameSpace, tagName);
                return patchElem(tagName, elem);
            };
        }
        if (props.tagPatchStrategy == 'prototype') {
            function patchProperties(prototype, properties) {
                const GetAttributeOriginal = Symbol();
                prototype[GetAttributeOriginal] = prototype.getAttribute;
                prototype.getAttribute = function (qualifiedName) {
                    if (properties.has(qualifiedName.toLowerCase())) {
                        return (this[GetAttributeOriginal](`patched-${qualifiedName}`) ||
                            this[GetAttributeOriginal](qualifiedName));
                    }
                    else {
                        return this[GetAttributeOriginal](qualifiedName);
                    }
                };
                const GetAttributeNSOriginal = Symbol();
                prototype[GetAttributeNSOriginal] = prototype.getAttributeNS;
                prototype.getAttributeNS = function (qualifiedName, localName) {
                    if (properties.has(qualifiedName.toLowerCase())) {
                        return (this[GetAttributeNSOriginal](`patched-${qualifiedName}`, localName) ||
                            this[GetAttributeNSOriginal](qualifiedName, localName));
                    }
                    else {
                        return this[GetAttributeNSOriginal](qualifiedName, localName);
                    }
                };
                const GetAttributeNodeOriginal = Symbol();
                prototype[GetAttributeNodeOriginal] = prototype.getAttributeNode;
                prototype.getAttributeNode = function (qualifiedName) {
                    if (properties.has(qualifiedName.toLowerCase())) {
                        return (this[GetAttributeNodeOriginal](`patched-${qualifiedName}`) ||
                            this[GetAttributeNodeOriginal](qualifiedName));
                    }
                    else {
                        return this[GetAttributeNodeOriginal](qualifiedName);
                    }
                };
                const GetAttributeNodeNSOriginal = Symbol();
                prototype[GetAttributeNodeNSOriginal] = prototype.getAttributeNodeNS;
                prototype.getAttributeNodeNS = function (qualifiedName, localName) {
                    if (properties.has(qualifiedName.toLowerCase())) {
                        return (this[GetAttributeNodeNSOriginal](`patched-${qualifiedName}`, localName) ||
                            this[GetAttributeNodeNSOriginal](qualifiedName, localName));
                    }
                    else {
                        return this[GetAttributeNodeNSOriginal](qualifiedName, localName);
                    }
                };
                const SetAttributeOriginal = Symbol();
                prototype[SetAttributeOriginal] = prototype.setAttribute;
                prototype.setAttribute = function (qualifiedName, value) {
                    if (properties.has(qualifiedName.toLowerCase())) {
                        this[SetAttributeOriginal](qualifiedName, getResourceUrl(value, context));
                        this[SetAttributeOriginal](`patched-${qualifiedName}`, value);
                    }
                    else {
                        this[SetAttributeOriginal](qualifiedName, value);
                    }
                };
                const SetAttributeNSOriginal = Symbol();
                prototype[SetAttributeNSOriginal] = prototype.setAttributeNS;
                prototype.setAttributeNS = function (namespace, qualifiedName, value) {
                    if (properties.has(qualifiedName.toLowerCase())) {
                        this[SetAttributeNSOriginal](namespace, qualifiedName, getResourceUrl(value, context));
                        this[SetAttributeNSOriginal](namespace, `patched-${qualifiedName}`, value);
                    }
                    else {
                        this[SetAttributeNSOriginal](namespace, qualifiedName, value);
                    }
                };
                const SetAttributeNodeOriginal = Symbol();
                prototype[SetAttributeNodeOriginal] = prototype.setAttributeNode;
                prototype.setAttributeNode = function (attr) {
                    if (properties.has(attr.name.toLowerCase())) {
                        const patchedAttr = this.ownerDocument.createAttribute(`patched-${attr.name}`);
                        patchedAttr.value = attr.value;
                        const newAttr = this.ownerDocument.createAttribute(attr.name);
                        newAttr.value = getResourceUrl(attr.value, context);
                        this[SetAttributeNodeOriginal](patchedAttr);
                        return this[SetAttributeNodeOriginal](newAttr);
                    }
                    else {
                        return this[SetAttributeNodeOriginal](attr);
                    }
                };
                const SetAttributeNodeNSOriginal = Symbol();
                prototype[SetAttributeNodeNSOriginal] = prototype.setAttributeNodeNS;
                prototype.setAttributeNodeNS = function (attr) {
                    if (properties.has(attr.name.toLowerCase())) {
                        const patchedAttr = this.ownerDocument.createAttributeNS(attr.namespaceURI, `patched-${attr.name}`);
                        patchedAttr.value = attr.value;
                        const newAttr = this.ownerDocument.createAttributeNS(attr.namespaceURI, attr.name);
                        newAttr.value = getResourceUrl(attr.value, context);
                        this[SetAttributeNodeNSOriginal](patchedAttr);
                        return this[SetAttributeNodeNSOriginal](newAttr);
                    }
                    else {
                        return this[SetAttributeNodeNSOriginal](attr);
                    }
                };
                for (const property of properties) {
                    Object.defineProperty(prototype, property, {
                        configurable: true,
                        enumerable: true,
                        get() {
                            return this.getAttribute(property);
                        },
                        set(v) {
                            return this.setAttribute(property, v);
                        }
                    });
                }
            }
            const win = iframe.contentWindow;
            patchProperties(win.HTMLLinkElement.prototype, new Set(['href']));
            patchProperties(win.HTMLScriptElement.prototype, new Set(['src']));
            patchProperties(win.HTMLImageElement.prototype, new Set(['src']));
        }
    }
    function patchIframeDocumentQueries(iframe) {
        const framedoc = iframe.contentWindow.document;
        const querySelector = framedoc.querySelector.bind(framedoc);
        framedoc.querySelector = selectors => {
            return querySelector(selectors.replaceAll('href', 'patched-href').replaceAll('src', 'patched-src'));
        };
    }
    function patchIframeFetch(iframe, contextUrl) {
        const base = (requestInfo, init) => fetchUrlContent(typeof requestInfo == 'string'
            ? mkUrl(contextUrl, requestInfo).href
            : Object.assign(Object.assign({}, requestInfo), { url: mkUrl(contextUrl, requestInfo.url).href }), init);
        if (props.fetchProxy) {
            iframe.contentWindow.fetch = (requestInfo, init) => props.fetchProxy({ requestInfo, init, contextUrl, base });
            return;
        }
        iframe.contentWindow.fetch = base;
        return;
    }
    function patchIframeWebSocket(iframe) {
        iframe.contentWindow.WebSocket = WebSocket;
    }
    function makeWorkerFromString(str) {
        return new Worker('data:application/javascript,' + encodeURIComponent(str));
    }
    function makePatchedWorker(iframe, contextUrl) {
        return class PatchedWorker extends Worker {
            constructor(scriptURL, options) {
                const url = getResourceUrl(scriptURL, contextUrl);
                super(url, options);
                const patchedWorkerPromise = (() => __awaiter(this, void 0, void 0, function* () {
                    const response = yield fetch(url);
                    const code = yield response.text();
                    const worker = makeWorkerFromString(`
fetchCallbacks = {};
fetch=async (resource, init)=>{
    const id = \`\${Math.random()}\`.substr(2);
    const promise = new Promise(res=>{fetchCallbacks[id]=res;});
    self.postMessage({isFetch:true, id, resource, init});
    return await promise;
};
self.addEventListener("message", function(event) {
    if(event.data.isFetchResult) {
        fetchCallbacks[event.data.id](new Response(event.data.blob, event.data.init));
        fetchCallbacks[event.data.id] = null;
    }
});
${code}`);
                    worker.addEventListener('message', function (event) {
                        return __awaiter(this, void 0, void 0, function* () {
                            if (event.data.isFetch) {
                                const response = yield iframe.contentWindow.fetch(event.data.resource, event.data.init);
                                const blob = yield response.blob();
                                worker.postMessage({
                                    isFetchResult: true,
                                    id: event.data.id,
                                    blob,
                                    init: {
                                        status: response.status,
                                        statusText: response.statusText,
                                        headers: response.headers
                                    }
                                });
                            }
                        });
                    });
                    return worker;
                }))();
                return new Proxy(this, {
                    get(target, propKey, receiver) {
                        const propValue = target[propKey];
                        if (typeof propValue != 'function') {
                            return propValue;
                        }
                        else {
                            return function (...args) {
                                return __awaiter(this, void 0, void 0, function* () {
                                    const patchedWorker = yield patchedWorkerPromise;
                                    return patchedWorker[propKey](...args);
                                });
                            };
                        }
                    }
                });
            }
        };
    }
    function patchIframeWorker(iframe, contextUrl) {
        iframe.contentWindow.Worker = makePatchedWorker(iframe, contextUrl);
    }
    function patchIframeXMLHttpRequest(iframe, contextUrl) {
        const base = (requestInfo, init) => {
            return fetchUrlContent(typeof requestInfo == 'string'
                ? mkUrl(contextUrl, requestInfo).href
                : Object.assign(Object.assign({}, requestInfo), { url: mkUrl(contextUrl, requestInfo.url).href }), init);
        };
        let f = base;
        if (props.fetchProxy) {
            f = (requestInfo, init) => {
                return props.fetchProxy({ requestInfo, init, contextUrl, base });
            };
        }
        const FXHR = createXMLHttpRequest();
        FXHR.addHandler({
            url: /.*/,
            headers: function (request, url, data) {
                return __awaiter(this, void 0, void 0, function* () {
                    const result = yield f(url, Object.assign({ method: request.method }, (data ? { body: data } : {})));
                    request._tmp = request._tmp || {};
                    if (request.responseType == 'arraybuffer') {
                        request._tmp.response = yield result.arrayBuffer();
                    }
                    else {
                        request._tmp.response = yield result.text();
                    }
                    return Object.fromEntries(result.headers.entries());
                });
            },
            status: 200,
            statusText: 'OK',
            response: function (request, url, data) {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    return (_a = request === null || request === void 0 ? void 0 : request._tmp) === null || _a === void 0 ? void 0 : _a.response;
                });
            }
        });
        iframe.contentWindow.XMLHttpRequest = FXHR;
    }
    function patchCustomDom(customDom) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!patchedElements.has(customDom)) {
                patchedElements.add(customDom);
                addCustomDomMutationObserver(customDom);
            }
        });
    }
    function patchIframe(iframe) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!patchedElements.has(iframe) ||
                (iframe.getAttribute('srcDoc') && iframe.getAttribute('srcDoc') != patchedElementSrcDocs.get(iframe))) {
                patchedElements.add(iframe);
                const src = iframe.getAttribute('src') || iframe.getAttribute('patched-src');
                yield setIframeSrc(src);
                function setIframeSrc(src) {
                    return __awaiter(this, void 0, void 0, function* () {
                        let newSrc;
                        let content;
                        if (src) {
                            iframe.setAttribute('patched-src', src);
                            iframe.removeAttribute('src');
                            newSrc = proxySrc(src);
                            content = yield (yield fetchUrlContent(newSrc, {
                                headers: {
                                    Accept: `text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8`,
                                    'Accept-Encoding': `gzip, deflate, br`
                                }
                            })).text();
                        }
                        else {
                            src = tryGetIframeContext(iframe);
                            content = iframe.getAttribute('srcDoc');
                            patchedElementSrcDocs.set(iframe, content);
                            // iframe.removeAttribute('srcDoc');
                        }
                        const { html, context } = yield patchHtmlCode(content, src);
                        patchIframeTags(iframe, context);
                        patchIframeEventListeners(iframe, context);
                        patchIframeClasses(iframe);
                        patchIframePostMessage(iframe);
                        patchIframeFetch(iframe, context);
                        patchIframeConsole(iframe);
                        patchIframeWorker(iframe, context);
                        patchIframeXMLHttpRequest(iframe, context);
                        patchIframeWebSocket(iframe);
                        setIframeContent(iframe, html);
                        addIframeMutationObserverWhenReady(iframe);
                        iframe.setAttribute('patched', 'true');
                        yield props.onIframePatch(iframe);
                    });
                }
                const oldSetAttribute = iframe.setAttribute.bind(iframe);
                iframe.setAttribute = (qualifiedName, value) => {
                    if (qualifiedName == 'src') {
                        setIframeSrc(value);
                    }
                    else {
                        oldSetAttribute(qualifiedName, value);
                    }
                };
                const oldSetAttributeNS = iframe.setAttributeNS.bind(iframe);
                iframe.setAttributeNS = (namespace, qualifiedName, value) => {
                    if (qualifiedName == 'src') {
                        setIframeSrc(value);
                    }
                    else {
                        oldSetAttributeNS(namespace, qualifiedName, value);
                    }
                };
            }
        });
    }
    function patchIframes(iframes) {
        [...iframes].forEach(patchIframe);
    }
    function patchCustomDoms(customDoms) {
        [...customDoms].forEach(patchCustomDom);
    }
    function addIframeMutationObserverWhenReady(iframe) {
        iframe.addEventListener('load', function (e) {
            addIframeMutationObserver(e);
        });
        addIframeMutationObserver(iframe);
    }
    function mutationObserverCallback(records) {
        const iframes = records.map(x => x.target).filter(x => x.tagName == 'IFRAME');
        if (iframes.length > 0) {
            patchIframes(iframes);
        }
        const nodes = records.map(x => x.target).concat(records.flatMap(x => [...x.addedNodes]));
        const shadowDoms = nodes.filter(x => x.shadowRoot);
        if (shadowDoms.length > 0) {
            patchCustomDoms(shadowDoms);
        }
        const linkTags = nodes.filter(x => x.tagName == 'LINK');
        if (linkTags > 0) {
            const context = tryGetIframeContext(linkTags[0]);
            linkTags.forEach(t => patchLinkTag(t, context));
        }
    }
    const iframeObservers = new WeakMap();
    function addCustomDomMutationObserver(customDom) {
        if (!customDom.shadowRoot) {
            return;
        }
        const documentElement = customDom.shadowRoot.getRootNode();
        if (!iframeObservers.has(documentElement)) {
            const mutationObserver = new MutationObserver(mutationObserverCallback);
            mutationObserver.observe(documentElement, { childList: true, subtree: true, attributes: true });
            iframeObservers.set(documentElement, mutationObserver);
        }
        patchIframes(documentElement.querySelectorAll('iframe'));
        patchCustomDoms([...documentElement.querySelectorAll('*')].filter(x => x.shadowRoot));
    }
    function addIframeMutationObserver(iframe) {
        if (!iframe.contentWindow) {
            return;
        }
        const patchedSrc = iframe.getAttribute('patched-src');
        if (patchedSrc) {
            const patchedSrcUrl = new URL(patchedSrc);
            iframe.contentWindow.location.hash = patchedSrcUrl.hash;
        }
        const documentElement = iframe.contentWindow.document.documentElement;
        patchIframeDocumentQueries(iframe);
        if (!iframeObservers.has(documentElement)) {
            const mutationObserver = new MutationObserver(mutationObserverCallback);
            mutationObserver.observe(documentElement, { childList: true, subtree: true, attributes: true });
            iframeObservers.set(documentElement, mutationObserver);
        }
        patchIframes(documentElement.getElementsByTagName('iframe'));
        patchCustomDoms([...documentElement.getElementsByTagName('*')].filter(x => x.shadowRoot));
    }
    function setIframeContent(iframe, content) {
        const window = iframe.contentWindow;
        const doc = window.document;
        if (props.htmlPostProcessFunction) {
            content = props.htmlPostProcessFunction(content);
        }
        doc.open('text/html', 'replace');
        doc.write(content);
        doc.close();
    }
    function setIframeContentAndPatch(iframe, content) {
        setIframeContent(iframe, content);
        addIframeMutationObserverWhenReady(iframe);
    }
    useEffect(() => {
        const iframe = frame.current;
        if (!frame.current)
            return;
        setIframeContentAndPatch(iframe, `<style>body{margin:0px;}</style><iframe patched-src="${props.src}" width="100%" height="100%" allowfullscreen="allowfullscreen" frameborder="0">`);
        if (props.onload) {
            props.onload(iframe.contentDocument.body.firstChild);
        }
        return () => {
            // frame.current?.remove();
        };
    }, [frame]);
    useEffect(() => {
        return () => {
            var _a, _b, _c, _d;
            mockServers.forEach(mockServer => {
                mockServer.stop();
                mockServer.close();
            });
            mockServers = [];
            (_d = (_c = (_b = (_a = frame.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.location) === null || _c === void 0 ? void 0 : _c.reload) === null || _d === void 0 ? void 0 : _d.call(_c);
            // frame.current?.remove?.();
        };
    }, []);
    return React.createElement("iframe", Object.assign({ ref: frame, width: "100%", height: "100%", allowFullScreen: true }, props.outerIframeProps));
};

const OfflineIframe = (_a) => {
    var { address, fetch, getUrl } = _a, props = __rest(_a, ["address", "fetch", "getUrl"]);
    const LocalIframe = defineLocalIframe({ fetch, getUrl });
    return (React.createElement(LocalIframe, Object.assign({ onload: () => __awaiter(void 0, void 0, void 0, function* () { }), src: address, proxy: getUrl, fetchProxy: ({ requestInfo, base, contextUrl, init }) => __awaiter(void 0, void 0, void 0, function* () {
            requestInfo =
                typeof requestInfo == 'string'
                    ? getUrl(mkUrl(contextUrl, requestInfo).href)
                    : Object.assign(Object.assign({}, requestInfo), { url: getUrl(mkUrl(contextUrl, requestInfo.url).href) });
            return yield base(requestInfo, init);
        }), onIframePatch: () => __awaiter(void 0, void 0, void 0, function* () { }), tagPatchStrategy: 'prototype', postMessagePatchStrategy: 'target', onMessagePatchStrategy: null }, props)));
};

export { OfflineIframe };
//# sourceMappingURL=index.esm.js.map
